@@ -13,12 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import re
 import traceback
 
 from oslo_log import log
 from tempest.lib.cli import base
+from tempest.lib.common.utils import data_utils
 from tempest.lib import exceptions as lib_exc
 
+from manilaclient.common import constants
 from manilaclient import config
 from manilaclient.tests.functional import client
 from manilaclient.tests.functional import utils
@@ -40,14 +43,12 @@ class handle_cleanup_exceptions(object):
         return self
 
     def __exit__(self, exc_type, exc_value, exc_traceback):
-        if not (isinstance(exc_value,
-                           (lib_exc.NotFound, lib_exc.Forbidden)) or
-                CONF.suppress_errors_in_cleanup):
-            return False  # Do not suppress error if any
-        if exc_traceback:
-            LOG.error("Suppressed cleanup error: "
-                      "\n%s" % traceback.format_exc())
-        return True  # Suppress error if any
+        if isinstance(exc_value, (lib_exc.NotFound, lib_exc.Forbidden)):
+            return True
+        elif CONF.suppress_errors_in_cleanup:
+            LOG.error("Suppressed cleanup error: \n%s", traceback.format_exc())
+            return True
+        return False  # Don't suppress cleanup errors
 
 
 class BaseTestCase(base.ClientTestBase):
@@ -110,9 +111,14 @@ class BaseTestCase(base.ClientTestBase):
                             res_id, microversion=res["microversion"])
                         client.wait_for_snapshot_deletion(
                             res_id, microversion=res["microversion"])
+                    elif res["type"] is "share_replica":
+                        client.delete_share_replica(
+                            res_id, microversion=res["microversion"])
+                        client.wait_for_share_replica_deletion(
+                            res_id, microversion=res["microversion"])
                     else:
                         LOG.warning("Provided unsupported resource type for "
-                                    "cleanup '%s'. Skipping." % res["type"])
+                                    "cleanup '%s'. Skipping.", res["type"])
                 res["deleted"] = True
 
     @classmethod
@@ -121,6 +127,10 @@ class BaseTestCase(base.ClientTestBase):
             username=CONF.admin_username,
             password=CONF.admin_password,
             tenant_name=CONF.admin_tenant_name,
+            project_domain_name=CONF.admin_project_domain_name or None,
+            project_domain_id=CONF.admin_project_domain_id or None,
+            user_domain_name=CONF.admin_user_domain_name or None,
+            user_domain_id=CONF.admin_user_domain_id or None,
             uri=CONF.admin_auth_url or CONF.auth_url,
             insecure=CONF.insecure,
             cli_dir=CONF.manila_exec_dir)
@@ -134,6 +144,10 @@ class BaseTestCase(base.ClientTestBase):
             username=CONF.username,
             password=CONF.password,
             tenant_name=CONF.tenant_name,
+            project_domain_name=CONF.project_domain_name or None,
+            project_domain_id=CONF.project_domain_id or None,
+            user_domain_name=CONF.user_domain_name or None,
+            user_domain_id=CONF.user_domain_id or None,
             uri=CONF.auth_url,
             insecure=CONF.insecure,
             cli_dir=CONF.manila_exec_dir)
@@ -163,17 +177,28 @@ class BaseTestCase(base.ClientTestBase):
 
     @classmethod
     def create_share_type(cls, name=None, driver_handles_share_servers=True,
-                          snapshot_support=True, is_public=True, client=None,
-                          cleanup_in_class=True, microversion=None):
+                          snapshot_support=None,
+                          create_share_from_snapshot=None,
+                          revert_to_snapshot=None, mount_snapshot=None,
+                          is_public=True, client=None, cleanup_in_class=True,
+                          microversion=None, extra_specs=None,
+                          description=None):
         if client is None:
             client = cls.get_admin_client()
-        share_type = client.create_share_type(
-            name=name,
-            driver_handles_share_servers=driver_handles_share_servers,
-            snapshot_support=snapshot_support,
-            is_public=is_public,
-            microversion=microversion,
-        )
+        data = {
+            "name": name,
+            "driver_handles_share_servers": driver_handles_share_servers,
+            "snapshot_support": snapshot_support,
+            "is_public": is_public,
+            "microversion": microversion,
+            "extra_specs": extra_specs,
+            "create_share_from_snapshot": create_share_from_snapshot,
+            "revert_to_snapshot": revert_to_snapshot,
+            "mount_snapshot": mount_snapshot,
+        }
+        if description:
+            data["description"] = description
+        share_type = client.create_share_type(**data)
         resource = {
             "type": "share_type",
             "id": share_type["ID"],
@@ -188,7 +213,7 @@ class BaseTestCase(base.ClientTestBase):
 
     @classmethod
     def create_share_network(cls, name=None, description=None,
-                             nova_net_id=None, neutron_net_id=None,
+                             neutron_net_id=None,
                              neutron_subnet_id=None, client=None,
                              cleanup_in_class=True, microversion=None):
         if client is None:
@@ -196,7 +221,6 @@ class BaseTestCase(base.ClientTestBase):
         share_network = client.create_share_network(
             name=name,
             description=description,
-            nova_net_id=nova_net_id,
             neutron_net_id=neutron_net_id,
             neutron_subnet_id=neutron_subnet_id,
             microversion=microversion,
@@ -219,8 +243,7 @@ class BaseTestCase(base.ClientTestBase):
                      public=False, snapshot=None, metadata=None,
                      client=None, cleanup_in_class=False,
                      wait_for_creation=True, microversion=None):
-        if client is None:
-            client = cls.get_admin_client()
+        client = client or cls.get_admin_client()
         data = {
             'share_protocol': share_protocol or client.share_protocol,
             'size': size or 1,
@@ -231,12 +254,13 @@ class BaseTestCase(base.ClientTestBase):
             'metadata': metadata,
             'microversion': microversion,
         }
-        share_network = share_network or client.share_network
+
         share_type = share_type or CONF.share_type
-        if share_network:
-            data['share_network'] = share_network
-        if share_type:
-            data['share_type'] = share_type
+        share_network = share_network or cls._determine_share_network_to_use(
+            client, share_type, microversion=microversion)
+
+        data['share_type'] = share_type
+        data['share_network'] = share_network
         share = client.create_share(**data)
         resource = {
             "type": "share",
@@ -249,12 +273,25 @@ class BaseTestCase(base.ClientTestBase):
         else:
             cls.method_resources.insert(0, resource)
         if wait_for_creation:
-            client.wait_for_share_status(share['id'], 'available')
+            client.wait_for_resource_status(share['id'],
+                                            constants.STATUS_AVAILABLE)
         return share
 
+    @classmethod
+    def _determine_share_network_to_use(cls, client, share_type,
+                                        microversion=None):
+        """Determine what share network we need from the share type."""
+
+        # Get share type, determine if we need the share network
+        share_type = client.get_share_type(share_type,
+                                           microversion=microversion)
+        dhss_pattern = re.compile('driver_handles_share_servers : ([a-zA-Z]+)')
+        dhss = dhss_pattern.search(share_type['required_extra_specs']).group(1)
+        return client.share_network if dhss.lower() == 'true' else None
+
     @classmethod
     def create_security_service(cls, type='ldap', name=None, description=None,
-                                dns_ip=None, server=None, domain=None,
+                                dns_ip=None, ou=None, server=None, domain=None,
                                 user=None, password=None, client=None,
                                 cleanup_in_class=False, microversion=None):
         if client is None:
@@ -268,6 +305,7 @@ class BaseTestCase(base.ClientTestBase):
             'server': server,
             'domain': domain,
             'dns_ip': dns_ip,
+            'ou': ou,
             'microversion': microversion,
         }
         ss = client.create_security_service(**data)
@@ -310,3 +348,64 @@ class BaseTestCase(base.ClientTestBase):
         if wait_for_creation:
             client.wait_for_snapshot_status(snapshot['id'], 'available')
         return snapshot
+
+    @classmethod
+    def create_message(cls, client=None, wait_for_creation=True,
+                       cleanup_in_class=False, microversion=None):
+        """Trigger a 'no valid host' situation to generate a message."""
+        if client is None:
+            client = cls.get_admin_client()
+
+        extra_specs = {
+            'vendor_name': 'foobar',
+        }
+        share_type_name = data_utils.rand_name("share-type")
+        cls.create_share_type(
+            name=share_type_name, extra_specs=extra_specs,
+            driver_handles_share_servers=False, client=client,
+            cleanup_in_class=cleanup_in_class, microversion=microversion)
+
+        share_name = data_utils.rand_name("share")
+        share = cls.create_share(
+            name=share_name, share_type=share_type_name,
+            cleanup_in_class=cleanup_in_class, microversion=microversion,
+            wait_for_creation=False, client=client)
+
+        client.wait_for_resource_status(share['id'], constants.STATUS_ERROR)
+        message = client.wait_for_message(share['id'])
+
+        resource = {
+            "type": "message",
+            "id": message["ID"],
+            "client": client,
+            "microversion": microversion,
+        }
+        if cleanup_in_class:
+            cls.class_resources.insert(0, resource)
+        else:
+            cls.method_resources.insert(0, resource)
+        return message
+
+    @classmethod
+    def create_share_replica(cls, share_id, client=None,
+                             wait_for_creation=True, cleanup_in_class=False,
+                             microversion=None):
+        client = client or cls.get_user_client()
+
+        share_replica = client.create_share_replica(
+            share_id, microversion=microversion)
+        if wait_for_creation:
+            share_replica = client.wait_for_share_replica_status(
+                share_replica['id'])
+
+        resource = {
+            "type": "share_replica",
+            "id": share_replica["id"],
+            "client": client,
+            "microversion": microversion,
+        }
+        if cleanup_in_class:
+            cls.class_resources.insert(0, resource)
+        else:
+            cls.method_resources.insert(0, resource)
+        return share_replica
